-- ============================================================= --
-- LUMBERJACK MOD
-- ============================================================= --
LumberJack = {};
source(g_currentModDirectory .. 'DeleteShapeEvent.lua');

addModEventListener(LumberJack);

-- ALLOW CHAINSAW CUTTING ANYWHERE ON THE MAP
function LumberJack:isCuttingAllowed(superFunc, x, y, z)
	return true;
end

-- ADD SHORTCUT KEY SELECTION TO OPTIONS MENU
function LumberJack:registerActionEvents()
	local _, actionEventId = g_inputBinding:registerActionEvent('LUMBERJACK_STRENGTH', self, LumberJack.toggleStrength, false, true, false, true);
	g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_LOW)
	g_inputBinding:setActionEventActive(actionEventId, true)
    g_inputBinding:setActionEventText(actionEventId, g_i18n:getText("menu_TOGGLE_STRENGTH"))
end;

function LumberJack:loadMap(name)
	--print("Load Mod: 'LumberJack'")
	self.setOnceFlag = false
	self.stumpGrindingTime = 0
	self.stumpGrindingFlag = false
	self.useChainsawFlag = false
	self.foundObjectFlag = false
	self.pickUpObjectFlag = false
	self.driveVehicleFlag = false
	self.hasHandtoolEquipped = false
	self.superStrength = false
	self.lastObjectMass = 0
	self.superStrengthValue = 100
	self.normalStrengthValue = 0.2
	self.ringSelector = 0
	self.splitShape = 0
	self.showDebug = false
	
	-- ALLOW CHAINSAW CUTTING ANYWHERE ON THE MAP
	Chainsaw.isCuttingAllowed = Utils.overwrittenFunction(Chainsaw.isCuttingAllowed, LumberJack.isCuttingAllowed);
	
	-- ADD SHORTCUT KEY SELECTION TO OPTIONS MENU
	Player.registerActionEvents = Utils.appendedFunction(Player.registerActionEvents, LumberJack.registerActionEvents);
end
function LumberJack:deleteMap()
end
function LumberJack:mouseEvent(posX, posY, isDown, isUp, button)
end
function LumberJack:keyEvent(unicode, sym, modifier, isDown)
end
function LumberJack:draw()
end

function LumberJack:toggleStrength()
	if g_currentMission.player.isEntered and not (LumberJack.pickUpObjectFlag or g_gui:getIsGuiVisible()) then
		if LumberJack.superStrength then
			--print("SUPER STRENGTH OFF")
			Player.MAX_PICKABLE_OBJECT_MASS = LumberJack.normalStrengthValue
			LumberJack.superStrength = false
		else
			--print("SUPER STRENGTH ON")
			Player.MAX_PICKABLE_OBJECT_MASS = LumberJack.superStrengthValue
			LumberJack.superStrength = true
		end
	end
end

function LumberJack:update(dt)
	-- CHANGE GLOBAL VALUES ON FIRST RUN
	if (g_gameStateManager:getGameState()==GameState.PLAY and self.setOnceFlag==false) then
		-- Only change values from default
		if Player.MAX_PICKABLE_OBJECT_DISTANCE == 3.00 then
			Player.MAX_PICKABLE_OBJECT_DISTANCE = 6.00
		end
		if g_currentMission.player.minCutDistance == 0.50 then
			g_currentMission.player.minCutDistance = 0.10
		end
		if g_currentMission.player.maxCutDistance == 2.00 then
			g_currentMission.player.maxCutDistance = 4.00
		end
		self.setOnceFlag = true
	end

	-- IF ENTERING A VEHICLE
	if g_currentMission.controlledVehicle then
		if not self.driveVehicleFlag then
			--print("PLAYER IS DRIVING")
			self.driveVehicleFlag = true
		end
	else
		self.driveVehicleFlag = false
	end
	
	-- IF OBSERVING AN OBJECT
	if g_currentMission.player.isObjectInRange then
		-- check if we saw this object last time
		if self.lastObjectMass ~= g_currentMission.player.lastFoundObjectMass then
			self.lastObjectMass = g_currentMission.player.lastFoundObjectMass
		end

		-- Display Mass of LAST OBSERVED OBJECT in 'F1' Help Menu
		if not (self.driveVehicleFlag or g_currentMission.player:hasHandtoolEquipped()) then
			g_currentMission:addExtraPrintText(string.format("Mass: %.1f kg", 1000*self.lastObjectMass))
		end

		self.foundObjectFlag = true
	else
		self.lastObjectMass = 0
		self.foundObjectFlag = false
	end

	-- IF ACTUALLY CARRYING AN OBJECT
	if g_currentMission.player.isCarryingObject then
		if not self.pickUpObjectFlag then
			-- adjust super strength when picking up an object
			if self.superStrength and 5*self.lastObjectMass > self.normalStrengthValue then
				Player.MAX_PICKABLE_OBJECT_MASS = 5*self.lastObjectMass
			end
			self.pickUpObjectFlag = true
		end
	else
	-- IF NOT CARRYING AN OBJECT
		if self.pickUpObjectFlag then
			-- Restore Full super strength when dropping an object
			if self.superStrength then
				Player.MAX_PICKABLE_OBJECT_MASS = self.superStrengthValue
			end
			
			self.pickUpObjectFlag = false
		end
	end
	
	-- CHANGE COLOUR OF THE CURSOR/HAND ICON
	if not self.driveVehicleFlag then
		if self.superStrength then -- Make hand BRIGHTER when super strength is ON
			g_currentMission.player.pickedUpObjectOverlay:setColor(1, 1, 1, 1.0)
		elseif self.lastObjectMass>Player.MAX_PICKABLE_OBJECT_MASS then
			-- Make hand RED when objects are too heavy to pick up
			g_currentMission.player.pickedUpObjectOverlay:setColor(1.0, 0.1, 0.1, 0.5)
		else -- Make hand GREY for everything else
			g_currentMission.player.pickedUpObjectOverlay:setColor(1, 1, 1, 0.3)
		end
	end
	
	-- DESTROY SMALL LOGS WHEN USING THE CHAINSAW --
	if g_currentMission.player:hasHandtoolEquipped() then
		self.hasHandtoolEquipped = true
		local hTool = g_currentMission.player.baseInformation.currentHandtool
		
		if hTool ~= nil then
			if self.ringSelector == 0 then
				-- CREATE EXTRA RING SELECTOR
				local hTool = g_currentMission.player.baseInformation.currentHandtool
				local filename = 'data/firstPerson/chainsaws/chainsawRingSelector.i3d'
				local i3dNode = g_i3DManager:loadSharedI3DFile(filename, hTool.baseDirectory, false, false, false)
				if i3dNode ~= 0 then
					self.ringSelector = getChildAt(i3dNode, 0)
					setVisibility(self.ringSelector, true)
					setShaderParameter(self.ringSelector, "colorScale", 0.05, 0.05, 0.05, 1.0, false)
					link(g_currentMission.player.chainsawCameraFocus, self.ringSelector)
					--setTranslation(self.ringSelector, 0,0,-0.3)
					delete(i3dNode)
				end
			end
			-- INCRESE CUTTING SPEED x2 (default value is 8.0)
			hTool.defaultCutDuration = 4.0
			
			-- DESTROY SMALL LOGS WHEN USING THE CHAINSAW --
			if hTool.isCutting then
				--print("CHAINSAW CUTTING")				
				if self.ringSelector ~= 0 then
					setVisibility(self.ringSelector, false)
				end
				if not self.useChainsawFlag then
					-- Find the splitShape from chainsawSplitShapeFocus (lastFoundObject doesn't exist for client in multiplayer)
					local x,y,z = getWorldTranslation(g_currentMission.player.chainsawSplitShapeFocus)
					local nx,ny,nz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 1,0,0)
					local yx,yy,yz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 0,1,0)
					local splitShape, minY, maxY, minZ, maxZ = findSplitShape(x,y,z, nx,ny,nz, yx,yy,yz, 5, 5)
					if splitShape ~=0 then
						if getVolume(splitShape) < 0.100 then
						-- DELETE THE SHAPE if too small to worry about (e.g. felling wedge or thin branch)
							if g_currentMission.missionDynamicInfo.isMultiplayer then
							--print("MULTIPLAYER")
								DeleteShapeEvent.sendEvent(splitShape)
							else
							--print("SINGLE PLAYER")
								delete(splitShape)
							end
						end
					end
					self.useChainsawFlag = true
				end
			else
				--print("CHAINSAW NOT CUTTING")				
				if hTool.ringSelector ~= nil and hTool.ringSelector ~= 0 then
					if getVisibility(hTool.ringSelector) then
						setVisibility(self.ringSelector, false)
					else
						setVisibility(hTool.ringSelector, false)
						setVisibility(self.ringSelector, true)
						-- Find the splitShape from chainsawSplitShapeFocus
						self.splitShape = 0
						local x,y,z = getWorldTranslation(g_currentMission.player.chainsawSplitShapeFocus)
						local xx,xy,xz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 1,0,0)
						local yx,yy,yz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 0,1,0)
						local zx,zy,zz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 0,0,1)
						--local zx,zy,zz = MathUtil.crossProduct(xx,xy,xz,yx,yy,yz)
						--local size_y = hTool.cutSizeY = 1.1 --local size_z = hTool.cutSizeZ = 1.0
						local size = 1.5

						local x0 = x + yx*0.45 + zx*0.3
						local y0 = y + yy*0.45 + zy*0.3
						local z0 = z + yz*0.45 + zz*0.3
						local xx0,xy0,xz0 = 1,0,0
						local yx0,yy0,yz0 = 0,-1,0
						local zx0,zy0,zz0 = 0,0,-1
						if self.showDebug then
							--print(" ")
							--print(string.format("(%s, %s, %s)", tostring(xx),tostring(yy),tostring(zz)))
							--print(string.format("(%s, %s, %s)", tostring(xx0),tostring(yy0),tostring(zz0)))
							local r = hTool.ringSelectorScaleOffset
							Utils.renderTextAtWorldPosition(x0-xx0*r,y0-xy0*r,z0-xz0*r, "-x", getCorrectTextSize(0.012), 0)
							Utils.renderTextAtWorldPosition(x0+xx0*r,y0+xy0*r,z0+xz0*r, "+x", getCorrectTextSize(0.012), 0)
							Utils.renderTextAtWorldPosition(x0-yx0*r,y0-yy0*r,z0-yz0*r, "-y", getCorrectTextSize(0.012), 0)
							Utils.renderTextAtWorldPosition(x0+yx0*r,y0+yy0*r,z0+yz0*r, "+y", getCorrectTextSize(0.012), 0)
							Utils.renderTextAtWorldPosition(x0-zx0*r,y0-zy0*r,z0-zz0*r, "-z", getCorrectTextSize(0.012), 0)
							Utils.renderTextAtWorldPosition(x0+zx0*r,y0+zy0*r,z0+zz0*r, "+z", getCorrectTextSize(0.012), 0)
						end

						x = x0 - xx0*size/2 + yx0*0.45 - zx0*size/2
						y = y0 - xy0*size/2 + yy0*0.45 - zy0*size/2
						z = z0 - xz0*size/2 + yz0*0.45 - zz0*size/2
						xx,xy,xz = xx0,xy0,xz0
						yx,yy,yz = yx0,yy0,yz0
						zx,zy,zz = zx0,zy0,zz0
						if self.showDebug then
							drawDebugLine(x,y,z,1,1,1,x+xx*size,y+xy*size,z+xz*size,1,1,1)
							drawDebugLine(x,y,z,1,1,1,x+zx*size,y+zy*size,z+zz*size,1,1,1)
							drawDebugLine(x+xx*size,y+xy*size,z+xz*size,1,1,1,x+xx*size+zx*size,y+xy*size+zy*size,z+xz*size+zz*size,1,1,1)
							drawDebugLine(x+zx*size,y+zy*size,z+zz*size,1,1,1,x+xx*size+zx*size,y+xy*size+zy*size,z+xz*size+zz*size,1,1,1)
						end
						if self.splitShape==0 then
							self.splitShape, _, _, _, _ = findSplitShape(x,y,z, -yx,-yy,-yz, xx,xy,xz, size, size)
							if self.splitShape~=0 then
							
								if self.superStrength then
									self.stumpGrindingFlag = true
								else
									local lenBelow, lenAbove = getSplitShapePlaneExtents(self.splitShape, x,y,z, -yx,-yy,-yz)
									local _,ly,_ = worldToLocal(self.splitShape, x,y,z)
									if ly < 0.5 and lenAbove < 1 then
										self.stumpGrindingFlag = true
									else
										self.stumpGrindingFlag = false
									end
									if self.showDebug then
										print(string.format("below:%s   above:%s   ly:%s", tostring(lenBelow),tostring(lenAbove),tostring(ly)))
									end
								end
							else
								self.stumpGrindingFlag = false
							end
						end

						if self.stumpGrindingFlag then
						-- SHOW RED RING SELECTOR
							setShaderParameter(self.ringSelector, "colorScale", 0.8, 0.05, 0.05, 1.0, false)
							setTranslation(self.ringSelector, 0,0,-0.3)
						else
						-- SHOW GREY RING SELECTOR
							setShaderParameter(self.ringSelector, "colorScale", 0.05, 0.05, 0.05, 1.0, false)
							setTranslation(self.ringSelector, 0,0,0)
						end
						
					end
				end
				
				-- GRIND STUMPS USING THE CHAINSAW --
				--if self.player.isClient and self.player.isEntered and not g_gui:getIsGuiVisible() then
				if self.stumpGrindingFlag and g_currentMission.player.isEntered and not g_gui:getIsGuiVisible()
				and hTool.speedFactor > 0.1 then
				--and Input.isMouseButtonPressed(Input.MOUSE_BUTTON_LEFT) then
					self.stumpGrindingTime = self.stumpGrindingTime + dt
					if self.stumpGrindingTime < 3000 then
						-- STUMP GRINDING
						g_currentMission.player:lockInput(true)
						local cutPosition = {getWorldTranslation(self.ringSelector)}
						local cutTranslation = {worldToLocal(getParent(hTool.graphicsNode), cutPosition[1], cutPosition[2], cutPosition[3])}
						setTranslation(hTool.graphicsNode, cutTranslation[1]/3, cutTranslation[2]/3, cutTranslation[3]/3)
						--setRotation(hTool.graphicsNode, math.random(-1, 1)*0.01, math.random(-1, 1)*0.01, math.random(-1, 1)*0.01)
						hTool.isCutting = true
						hTool:updateParticles()
						hTool.isCutting = false
					else
						-- DELETE THE SHAPE
						if g_currentMission.missionDynamicInfo.isMultiplayer then
						--print("MULTIPLAYER")
							DeleteShapeEvent.sendEvent(self.splitShape)
						else
						--print("SINGLE PLAYER")
							delete(self.splitShape)
						end
						self.splitShape = 0
						self.stumpGrindingTime = 0
						self.stumpGrindingFlag = false
						g_currentMission.player:lockInput(false)
					end
					
				else
					self.stumpGrindingTime = 0
					g_currentMission.player:lockInput(false)
				end

				self.useChainsawFlag = false
			end
		end
	else
		self.hasHandtoolEquipped = false
		if self.ringSelector ~= 0 then
			if getVisibility(self.ringSelector) then
				setVisibility(self.ringSelector, false)
			end
		end
	end
end

						-- -- Find the splitShape from chainsawSplitShapeFocus
						-- self.splitShape = 0
						-- local x,y,z = getWorldTranslation(g_currentMission.player.chainsawSplitShapeFocus)
						-- local xx,xy,xz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 1,0,0)
						-- local yx,yy,yz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 0,1,0)
						-- local zx,zy,zz = localDirectionToWorld(g_currentMission.player.chainsawSplitShapeFocus, 0,0,1)
						-- --local zx,zy,zz = MathUtil.crossProduct(xx,xy,xz,yx,yy,yz)
						-- --local size_y = hTool.cutSizeY = 1.1 --local size_z = hTool.cutSizeZ = 1.0
						-- local size = 1.5

						-- local x0 = x+yx*0.45
						-- local y0 = y+yy*0.45
						-- local z0 = z+yz*0.45
						-- if self.showDebug then
							-- local r = hTool.ringSelectorScaleOffset
							-- Utils.renderTextAtWorldPosition(x0-xx*r,y0-xy*r,z0-xz*r, "-x", getCorrectTextSize(0.012), 0)
							-- Utils.renderTextAtWorldPosition(x0+xx*r,y0+xy*r,z0+xz*r, "+x", getCorrectTextSize(0.012), 0)
							-- Utils.renderTextAtWorldPosition(x0-yx*r,y0-yy*r,z0-yz*r, "-y", getCorrectTextSize(0.012), 0)
							-- Utils.renderTextAtWorldPosition(x0+yx*r,y0+yy*r,z0+yz*r, "+y", getCorrectTextSize(0.012), 0)
							-- Utils.renderTextAtWorldPosition(x0-zx*r,y0-zy*r,z0-zz*r, "-z", getCorrectTextSize(0.012), 0)
							-- Utils.renderTextAtWorldPosition(x0+zx*r,y0+zy*r,z0+zz*r, "+z", getCorrectTextSize(0.012), 0)
						-- end
						
						-- --TRY XY PLANE:
						-- x = x0 - xx*size/2 - yx*size/2 + zx*size/2
						-- y = y0 - xy*size/2 - yy*size/2 + zy*size/2
						-- z = z0 - xz*size/2 - yz*size/2 + zz*size/2
						-- if self.showDebug then
							-- drawDebugLine(x,y,z,1,1,1,x+xx*size,y+xy*size,z+xz*size,1,1,1)
							-- drawDebugLine(x,y,z,1,1,1,x+yx*size,y+yy*size,z+yz*size,1,1,1)
							-- drawDebugLine(x+xx*size,y+xy*size,z+xz*size,1,1,1,x+xx*size+yx*size,y+xy*size+yy*size,z+xz*size+yz*size,1,1,1)
							-- drawDebugLine(x+yx*size,y+yy*size,z+yz*size,1,1,1,x+xx*size+yx*size,y+xy*size+yy*size,z+xz*size+yz*size,1,1,1)
						-- end
						-- if self.splitShape==0 then
							-- self.splitShape, _, _, _, _ = findSplitShape(x,y,z, zx,zy,zz, xx,xy,xz, size, size)
							-- if self.splitShape~=0 then
								-- local lenBelow, lenAbove = getSplitShapePlaneExtents(self.splitShape, x,y,z, zx,zy,zz)
								-- local _,ly,_ = worldToLocal(self.splitShape, x,y,z)
								-- --if (lenBelow <= 0.4 or ly < 0.21) and lenAbove < 1 then
									-- self.stumpGrindingFlag = true
								-- --end
							-- end
						-- end
						
						-- --OTHERWISE TRY XZ PLANE:
						-- x = x0 - xx*size/2
						-- y = y0 - xy*size/2
						-- z = z0 - xz*size/2
						-- if self.showDebug then
							-- drawDebugLine(x,y,z,1,1,1,x+xx*size,y+xy*size,z+xz*size,1,1,1)
							-- drawDebugLine(x,y,z,1,1,1,x+zx*size,y+zy*size,z+zz*size,1,1,1)
							-- drawDebugLine(x+xx*size,y+xy*size,z+xz*size,1,1,1,x+xx*size+zx*size,y+xy*size+zy*size,z+xz*size+zz*size,1,1,1)
							-- drawDebugLine(x+zx*size,y+zy*size,z+zz*size,1,1,1,x+xx*size+zx*size,y+xy*size+zy*size,z+xz*size+zz*size,1,1,1)
						-- end
						-- if self.splitShape==0 then
							-- self.splitShape, _, _, _, _ = findSplitShape(x,y,z, -yx,-yy,-yz, xx,xy,xz, size, size)
							-- if self.splitShape~=0 then
								-- local lenBelow, lenAbove = getSplitShapePlaneExtents(self.splitShape, x,y,z, -yx,-yy,-yz)
								-- local _,ly,_ = worldToLocal(self.splitShape, x,y,z)
								-- --if (lenBelow <= 0.4 or ly < 0.21) and lenAbove < 1 then
									-- self.stumpGrindingFlag = true
								-- --end
							-- end
						-- end
						
						-- --OTHERWISE TRY YZ PLANE:
						-- x = x0 - yx*size/2
						-- y = y0 - yy*size/2
						-- z = z0 - yz*size/2
						-- if self.showDebug then
							-- drawDebugLine(x,y,z,1,1,1,x+yx*size,y+yy*size,z+yz*size,1,1,1)
							-- drawDebugLine(x,y,z,1,1,1,x+zx*size,y+zy*size,z+zz*size,1,1,1)
							-- drawDebugLine(x+yx*size,y+yy*size,z+yz*size,1,1,1,x+yx*size+zx*size,y+yy*size+zy*size,z+yz*size+zz*size,1,1,1)
							-- drawDebugLine(x+zx*size,y+zy*size,z+zz*size,1,1,1,x+yx*size+zx*size,y+yy*size+zy*size,z+yz*size+zz*size,1,1,1)
						-- end
						-- if self.splitShape==0 then
							-- self.splitShape, _, _, _, _ = findSplitShape(x,y,z, xx,xy,xz, yx,yy,yz, size, size)
							-- if self.splitShape~=0 then
								-- local lenBelow, lenAbove = getSplitShapePlaneExtents(self.splitShape, x,y,z, xx,xy,xz)
								-- local _,ly,_ = worldToLocal(self.splitShape, x,y,z)
								-- --if (lenBelow <= 0.4 or ly < 0.21) and lenAbove < 1 then
									-- self.stumpGrindingFlag = true
								-- --end
							-- end
						-- end